import com.typesafe.config.{Config, ConfigFactory}
import scala.xml._
import java.nio.file.{Files, Path, Paths}
import scala.util.{Failure, Success, Try}
import xmlvalid._
import templates._

object Main {
  val conf: Config = ConfigFactory.load
  val debugMode: Boolean = conf.getBoolean("debug")

  implicit def templateFactory(source: Node): Template with Logger = {
    (source \ "trade").head.child
    .filter {
      case v: Elem => true
      case _ => false
    }(1).label match {
      case "repo" => new RepoTemplate(source) with Logger
      case v => throw new Exception("Undefined trade type")
    }
  }

  def main(args: Array[String]): Unit = {
    if (args.length != 2) println("Error: Undefined INPUT/OUTPUT directory. Usage: app INPUT OUTPUT")
    else if (!Files.exists(Paths.get(args(0)))) println("Error: INPUT directory doesn`t exist")
    else if (!Files.exists(Paths.get(args(1)))) println("Error: OUTPUT directory doesn`t exist")
    else {
      val p = new PrettyPrinter(140, 2)

      for (v <- Files.walk(Paths.get(args(0))).toArray) {
        val f = v.asInstanceOf[Path]
        if (!Files.isDirectory(f)) {
          val fName= f.getFileName.toString.split('.')
          val sourceXml = XML.loadFile(f.toString)
          val template = Template(sourceXml)
          val outputXml = XML.loadString(p.format(template.render))
          //val outputXml = template.template

          //template.log()

          Try {
            XMLValid("src/main/resources/nsd/nsd-ext-merged-schema.xsd").loadString(sourceXml.mkString)
          } match {
            case Success(_) => println("Source xml success check against NSD schema")
            case Failure(ex) => println(s"Source xml isn`t wellformed or invalid: ${ex.getMessage}")
          }

          Try {
            XMLValid("src/main/resources/rts/fpml-recordkeeping-merged-schema.xsd").loadString(outputXml.mkString)
          } match {
            case Success(_) => println("Output xml success check against RTS schema")
            case Failure(ex) => println(s"Output xml isn`t wellformed or invalid: ${ex.getMessage}")
          }

          XML.save(Paths.get(args(1), fName(0) + "_RTS." + fName(1)).toString, outputXml, xmlDecl = true)
        }
      }
    }
  }
}
